@@ -9,7 +9,6 @@
 from warnings import warn
 
 from .base import REGISTRY, Backend, BaseBackend, BasePyExtBackend, from_name, register
-from .cuda_backend import CudaBackend
 from .debug_backend import DebugBackend
 from .gtcpp_backend import GTCpuIfirstBackend, GTCpuKfirstBackend, GTGpuBackend
 from .module_generator import BaseModuleGenerator
@@ -22,7 +21,6 @@ __all__ = [
     "BaseBackend",
     "BaseModuleGenerator",
     "BasePyExtBackend",
-    "CudaBackend",
     "DebugBackend",
     "GTCpuIfirstBackend",
     "GTCpuKfirstBackend",
