@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 import typing
 
@@ -83,7 +77,6 @@ def _make_type_validator(t):
     if isinstance(t, type):
         return attr.validators.instance_of(t)
     else:
-        # isinstance(t, _TypeDescriptor)
         return t.validator
 
 
@@ -97,12 +90,12 @@ def _make_sequence_validator(type_list, container_types=(list, tuple)):
         try:
             assert isinstance(value, tuple(container_types))
             assert isinstance([item_validator(instance, attribute, v) for v in value], list)
-        except Exception:
+        except Exception as ex:
             raise ValueError(
                 "Expr ({value}) does not match the '{name}' specification".format(
                     value=value, name=attribute.name
                 )
-            )
+            ) from ex
 
     return _is_sequence_of_validator
 
@@ -126,12 +119,12 @@ def _make_dict_validator(type_list):
             assert isinstance(
                 [value_validator(instance, attribute, v) for v in value.values()], list
             )
-        except Exception:
+        except Exception as ex:
             raise ValueError(
                 "Expr ({value}) does not match the '{name}' specification".format(
                     value=value, name=attribute.name
                 )
-            )
+            ) from ex
 
     return _is_dict_of_validator
 
@@ -151,12 +144,12 @@ def _make_tuple_validator(type_list):
                 ],
                 list,
             )
-        except Exception:
+        except Exception as ex:
             raise ValueError(
                 "Expr ({value}) does not match the '{name}' specification".format(
                     value=value, name=attribute.name
                 )
-            )
+            ) from ex
 
     return _is_tuple_of_validator
 
@@ -172,7 +165,7 @@ def _make_union_validator(type_list):
                 validator(instance, attribute, value)
                 break
 
-            except Exception as e:
+            except Exception:
                 pass
         else:
             passed = False
@@ -224,7 +217,7 @@ def attribute(of, optional=False, **kwargs):
         attr_type_hint = of.type_hint
 
     elif isinstance(of, type):
-        # assert of in (bool, float, str, int, enum.Enum)
+        # assert of in (bool, float, str, int, enum.Enum) # noqa: ERA001 [commented-out-code]
         attr_validator = attr.validators.instance_of(of)
         attr_type_hint = of
 
@@ -234,22 +227,18 @@ def attribute(of, optional=False, **kwargs):
     if optional:
         attr_validator = attr.validators.optional(attr_validator)
         kwargs.setdefault("default", None)
-        # kwargs["kw_only"] = True
 
     return attr.ib(validator=attr_validator, type=attr_type_hint, **kwargs)
 
 
 class AttributeClassLike:
-    def validate(self):
-        ...
+    def validate(self): ...
 
     @property
-    def attributes(self):
-        ...
+    def attributes(self): ...
 
     @property
-    def as_dict(self):
-        ...
+    def as_dict(self): ...
 
 
 def attribclass(cls_or_none=None, **kwargs):
@@ -274,8 +263,9 @@ def attribclass(cls_or_none=None, **kwargs):
         for name, member in extra_members.items():
             if name in cls.__dict__.keys():
                 raise ValueError(
-                    "Name clashing with a existing '{name}' member"
-                    " of the decorated class ".format(name=name)
+                    "Name clashing with a existing '{name}' member of the decorated class ".format(
+                        name=name
+                    )
                 )
             setattr(cls, name, member)
 
