@@ -32,23 +32,20 @@ from gt4py.next.ffront.stages import (
     DSL_FOP,
     DSL_PRG,
     FOP,
-    PRG,
+    PAST_PRG,
 )
 from gt4py.next.iterator import ir as itir
 from gt4py.next.otf import arguments, stages, toolchain, workflow
 
 
-ARGS: typing.TypeAlias = arguments.JITArgs
-CARG: typing.TypeAlias = arguments.CompileTimeArgs
-IT_PRG: typing.TypeAlias = itir.Program
-
-
-INPUT_DATA: typing.TypeAlias = DSL_FOP | FOP | DSL_PRG | PRG | IT_PRG
-INPUT_PAIR: typing.TypeAlias = toolchain.CompilableProgram[INPUT_DATA, ARGS | CARG]
+IRDefinitionForm: typing.TypeAlias = DSL_FOP | FOP | DSL_PRG | PAST_PRG | itir.Program
+CompilableDefinition: typing.TypeAlias = toolchain.CompilableProgram[
+    IRDefinitionForm, arguments.JITArgs | arguments.CompileTimeArgs
+]
 
 
 @dataclasses.dataclass(frozen=True)
-class Transforms(workflow.MultiWorkflow[INPUT_PAIR, stages.CompilableProgram]):
+class Transforms(workflow.MultiWorkflow[CompilableDefinition, stages.CompilableProgram]):
     """
     Modular workflow for transformations with access to intermediates.
 
@@ -64,7 +61,8 @@ class Transforms(workflow.MultiWorkflow[INPUT_PAIR, stages.CompilableProgram]):
     """
 
     aotify_args: workflow.Workflow[
-        toolchain.CompilableProgram[INPUT_DATA, ARGS], toolchain.CompilableProgram[INPUT_DATA, CARG]
+        toolchain.CompilableProgram[IRDefinitionForm, arguments.JITArgs],
+        toolchain.CompilableProgram[IRDefinitionForm, arguments.CompileTimeArgs],
     ] = dataclasses.field(default_factory=arguments.adapted_jit_to_aot_args_factory)
 
     func_to_foast: workflow.Workflow[AOT_DSL_FOP, AOT_FOP] = dataclasses.field(
@@ -95,9 +93,9 @@ class Transforms(workflow.MultiWorkflow[INPUT_PAIR, stages.CompilableProgram]):
         default_factory=past_to_itir.past_to_gtir_factory
     )
 
-    def step_order(self, inp: INPUT_PAIR) -> list[str]:
+    def step_order(self, inp: CompilableDefinition) -> list[str]:
         steps: list[str] = []
-        if isinstance(inp.args, ARGS):
+        if isinstance(inp.args, arguments.JITArgs):
             steps.append("aotify_args")
         match inp.data:
             case DSL_FOP():
@@ -123,7 +121,7 @@ class Transforms(workflow.MultiWorkflow[INPUT_PAIR, stages.CompilableProgram]):
                 steps.extend(
                     ["func_to_past", "past_lint", "field_view_prog_args_transform", "past_to_itir"]
                 )
-            case PRG():
+            case PAST_PRG():
                 steps.extend(["past_lint", "field_view_prog_args_transform", "past_to_itir"])
             case itir.Program():
                 pass
@@ -144,25 +142,33 @@ class Backend(Generic[core_defs.DeviceTypeT]):
     name: str
     executor: workflow.Workflow[stages.CompilableProgram, stages.CompiledProgram]
     allocator: next_allocators.FieldBufferAllocatorProtocol[core_defs.DeviceTypeT]
-    transforms: workflow.Workflow[INPUT_PAIR, stages.CompilableProgram]
+    transforms: workflow.Workflow[CompilableDefinition, stages.CompilableProgram]
 
     def __call__(
         self,
-        program: INPUT_DATA,
+        program: IRDefinitionForm,
         *args: Any,
         **kwargs: Any,
     ) -> None:
-        if not isinstance(program, IT_PRG):
+        if not isinstance(program, itir.Program):
             args, kwargs = signature.convert_to_positional(program, *args, **kwargs)
-        self.jit(program, *args, **kwargs)(*args, **kwargs)
+        # TODO(egparedes): this extraction is not strictly correct, as we should only
+        #   extract values from the correct container types, not from ANY container,
+        #   but that would require a larger refactoring and anyway this Backend class
+        #   should be removed in the future.
+        extracted_args = tuple(arguments.extract(a) for a in args)
+        extracted_kwargs = {k: arguments.extract(v) for k, v in kwargs.items()}
+        self.jit(program, *args, **kwargs)(*extracted_args, **extracted_kwargs)
 
-    def jit(self, program: INPUT_DATA, *args: Any, **kwargs: Any) -> stages.CompiledProgram:
-        if not isinstance(program, IT_PRG):
+    def jit(self, program: IRDefinitionForm, *args: Any, **kwargs: Any) -> stages.CompiledProgram:
+        if not isinstance(program, itir.Program):
             args, kwargs = signature.convert_to_positional(program, *args, **kwargs)
         aot_args = arguments.CompileTimeArgs.from_concrete(*args, **kwargs)
         return self.compile(program, aot_args)
 
-    def compile(self, program: INPUT_DATA, compile_time_args: CARG) -> stages.CompiledProgram:
+    def compile(
+        self, program: IRDefinitionForm, compile_time_args: arguments.CompileTimeArgs
+    ) -> stages.CompiledProgram:
         return self.executor(
             self.transforms(toolchain.CompilableProgram(data=program, args=compile_time_args))
         )
