@@ -12,9 +12,10 @@ import dataclasses
 from typing import Optional, Sequence, TypeAlias
 
 import dace
-import sympy
 from dace import subsets as dace_subsets
 
+from gt4py import eve
+from gt4py.eve.extended_typing import MaybeNestedInTuple
 from gt4py.next import common as gtx_common
 from gt4py.next.iterator import ir as gtir
 from gt4py.next.iterator.ir_utils import common_pattern_matcher as cpm, domain_utils
@@ -41,64 +42,46 @@ FieldopDomain: TypeAlias = list[FieldopDomainRange]
 """Domain of a field operator represented as a list of `FieldopDomainRange` for each dimension."""
 
 
-def extract_domain(node: gtir.Expr) -> FieldopDomain:
+def get_field_domain(domain: domain_utils.SymbolicDomain) -> FieldopDomain:
     """
     Visits the domain of a field operator and returns a list of dimensions and
     the corresponding lower and upper bounds. The returned lower bound is inclusive,
-    the upper bound is exclusive: [lower_bound, upper_bound[
+    the upper bound is exclusive, i.e. `[lower_bound, upper_bound[`
+
+    Note that the doman dimensions are sorted in gt4py canonical order.
     """
-
-    domain = []
-
-    if cpm.is_call_to(node, ("cartesian_domain", "unstructured_domain")):
-        for named_range in node.args:
-            assert cpm.is_call_to(named_range, "named_range")
-            assert len(named_range.args) == 3
-            axis = named_range.args[0]
-            assert isinstance(axis, gtir.AxisLiteral)
-            lower_bound, upper_bound = (
-                gtir_to_sdfg_utils.get_symbolic(arg) for arg in named_range.args[1:3]
-            )
-            dim = gtx_common.Dimension(axis.value, axis.kind)
-            domain.append(FieldopDomainRange(dim, lower_bound, upper_bound))
-
-    elif isinstance(node, domain_utils.SymbolicDomain):
-        for dim, drange in node.ranges.items():
-            domain.append(
-                FieldopDomainRange(
-                    dim,
-                    gtir_to_sdfg_utils.get_symbolic(drange.start),
-                    gtir_to_sdfg_utils.get_symbolic(drange.stop),
-                )
-            )
-
-    else:
-        raise ValueError(f"Invalid domain {node}.")
-
-    return domain
-
-
-def simplify_domain_expr(expr: sympy.Basic, domain: FieldopDomain) -> dace.symbolic.SymbolicType:
-    """Simplifies a symbolic expression by applying constraints from domain range.
-
-    Dace uses sympy for symbolic expressions in the SDFG. By applying assumptions
-    on a sympy expression, we may obtain a simplified expression.
-    This is particularly important in the lowering of concat_where domain expressions,
-    because it usually results in cleaner memlet subsets and better map fusion.
-
-    Args:
-        expr: The symbolic expression to simplify.
-    Returns:
-        A new symbolic expression.
-    """
-    for dim_range in domain:
-        # We want to enforce the constraint `ub = lb + size`. The actual constraint
-        # is given by the assumption that the `size` variable is integer and non-negative.
-        size = sympy.var(f"__gtir_{dim_range.dim.value}_size", integer=True, negative=False)
-        expr = expr.subs(dim_range.start, dim_range.stop - size).subs(
-            size, dim_range.stop - dim_range.start
+    return [
+        FieldopDomainRange(
+            dim,
+            gtir_to_sdfg_utils.get_symbolic(domain.ranges[dim].start),
+            gtir_to_sdfg_utils.get_symbolic(domain.ranges[dim].stop),
         )
-    return dace.symbolic.simplify_ext(expr)
+        for dim in gtx_common.order_dimensions(domain.ranges.keys())
+    ]
+
+
+TargetDomain: TypeAlias = MaybeNestedInTuple[domain_utils.SymbolicDomain]
+"""Symbolic domain which defines the range to write in the target field.
+
+For tuple output in fieldview, `TargetDomain` is a tree-like tuple of symbolic domains.
+"""
+
+
+def extract_target_domain(node: gtir.Expr) -> TargetDomain:
+    """
+    Visit a GTIR domain expression and construct a `TargetDomain` symbolic domain.
+
+    We use a visitor class to extract the tree-like structure for (nested) tuple of domains.
+    """
+
+    class TargetDomainParser(eve.visitors.NodeTranslator):
+        def visit_FunCall(self, node: gtir.FunCall) -> TargetDomain:
+            if cpm.is_call_to(node, "make_tuple"):
+                return tuple(self.visit(arg) for arg in node.args)
+            else:
+                return domain_utils.SymbolicDomain.from_expr(node)
+
+    return TargetDomainParser().visit(node)
 
 
 def get_domain_indices(
@@ -129,7 +112,6 @@ def get_domain_indices(
 
 def get_field_layout(
     field_domain: FieldopDomain,
-    target_domain: FieldopDomain,
 ) -> tuple[list[gtx_common.Dimension], list[dace.symbolic.SymExpr], list[dace.symbolic.SymExpr]]:
     """
     Parse the field operator domain and generate the shape of the result field.
@@ -144,7 +126,6 @@ def get_field_layout(
 
     Args:
         field_domain: The field operator domain.
-        target_domain: Domain of the target field in the root `SetAt` expression.
 
     Returns:
         A tuple of three lists containing:
@@ -157,7 +138,7 @@ def get_field_layout(
     domain_dims = [domain_range.dim for domain_range in field_domain]
     domain_origin = [domain_range.start for domain_range in field_domain]
     domain_shape = [
-        simplify_domain_expr(sympy.Max(0, (domain_range.stop - domain_range.start)), target_domain)
+        dace.symbolic.pystr_to_symbolic(f"max(0, {domain_range.stop - domain_range.start})")
         for domain_range in field_domain
     ]
     return domain_dims, domain_origin, domain_shape
