@@ -22,16 +22,21 @@ This is likely to change in the future, to enable GTIR optimizations for scan.
 
 from __future__ import annotations
 
-import itertools
-from typing import Any, Iterable
+from typing import Iterable, Sequence
 
 import dace
 from dace import subsets as dace_subsets
 
 from gt4py import eve
+from gt4py.eve.extended_typing import MaybeNestedInTuple
 from gt4py.next import common as gtx_common, utils as gtx_utils
 from gt4py.next.iterator import ir as gtir
-from gt4py.next.iterator.ir_utils import common_pattern_matcher as cpm, ir_makers as im
+from gt4py.next.iterator.ir_utils import (
+    common_pattern_matcher as cpm,
+    domain_utils,
+    ir_makers as im,
+)
+from gt4py.next.iterator.transforms import infer_domain
 from gt4py.next.program_processors.runners.dace import (
     gtir_dataflow,
     gtir_domain,
@@ -46,8 +51,8 @@ def _parse_scan_fieldop_arg(
     node: gtir.Expr,
     ctx: gtir_to_sdfg.SubgraphContext,
     sdfg_builder: gtir_to_sdfg.SDFGBuilder,
-    domain: gtir_domain.FieldopDomain,
-) -> gtir_dataflow.MemletExpr | tuple[gtir_dataflow.MemletExpr | tuple[Any, ...], ...]:
+    field_domain: gtir_domain.FieldopDomain,
+) -> MaybeNestedInTuple[gtir_dataflow.MemletExpr]:
     """Helper method to visit an expression passed as argument to a scan field operator.
 
     On the innermost level, a scan operator is lowered to a loop region which computes
@@ -60,7 +65,7 @@ def _parse_scan_fieldop_arg(
     def _parse_fieldop_arg_impl(
         arg: gtir_to_sdfg_types.FieldopData,
     ) -> gtir_dataflow.MemletExpr:
-        arg_expr = arg.get_local_view(domain, ctx.sdfg)
+        arg_expr = arg.get_local_view(field_domain, ctx.sdfg)
         if isinstance(arg_expr, gtir_dataflow.MemletExpr):
             return arg_expr
         # In scan field operator, the arguments to the vertical stencil are passed by value.
@@ -78,17 +83,17 @@ def _parse_scan_fieldop_arg(
         return _parse_fieldop_arg_impl(arg)
     else:
         # handle tuples of fields
-        return gtx_utils.tree_map(lambda x: _parse_fieldop_arg_impl(x))(arg)
+        return gtx_utils.tree_map(_parse_fieldop_arg_impl)(arg)
 
 
 def _create_scan_field_operator_impl(
     ctx: gtir_to_sdfg.SubgraphContext,
     sdfg_builder: gtir_to_sdfg.SDFGBuilder,
-    field_domain: gtir_domain.FieldopDomain,
-    output_edge: gtir_dataflow.DataflowOutputEdge,
+    output_edge: gtir_dataflow.DataflowOutputEdge | None,
+    output_domain: infer_domain.NonTupleDomainAccess,
     output_type: ts.FieldType,
-    map_exit: dace.nodes.MapExit,
-) -> gtir_to_sdfg_types.FieldopData:
+    map_exit: dace.nodes.MapExit | None,
+) -> gtir_to_sdfg_types.FieldopData | None:
     """
     Helper method to allocate a temporary array that stores one field computed
     by the scan field operator.
@@ -101,9 +106,25 @@ def _create_scan_field_operator_impl(
     Therefore, the memlet subset will write a slice into the result array, that
     corresponds to the full vertical shape for each horizontal grid point.
 
+    Another difference is that this function is called on all fields inside a tuple,
+    in case of tuple return. Note that a regular field operator only computes a
+    single field, never a tuple of fields. For tuples, it can happen that one of
+    the nested fields is not used, outside the scan field operator, and therefore
+    does not need to be computed. Then, the domain inferred by gt4py on this field
+    is empty and the corresponding `output_edge` argument to this function is None.
+    In this case, the function does not allocate an array node for the output field
+    and returns None.
+
     Refer to `gtir_to_sdfg_primitives._create_field_operator_impl()` for
-    the description of function arguments and return values.
+    the description of function arguments.
     """
+    if output_edge is None:
+        # According to domain inference, this tuple field does not need to be computed.
+        assert output_domain == infer_domain.DomainAccessDescriptor.NEVER
+        return None
+    assert isinstance(output_domain, domain_utils.SymbolicDomain)
+    field_domain = gtir_domain.get_field_domain(output_domain)
+
     dataflow_output_desc = output_edge.result.dc_node.desc(ctx.sdfg)
     assert isinstance(dataflow_output_desc, dace.data.Array)
 
@@ -119,9 +140,7 @@ def _create_scan_field_operator_impl(
         raise NotImplementedError("scan with list output is not supported")
 
     # the memory layout of the output field follows the field operator compute domain
-    field_dims, field_origin, field_shape = gtir_domain.get_field_layout(
-        field_domain, ctx.target_domain
-    )
+    field_dims, field_origin, field_shape = gtir_domain.get_field_layout(field_domain)
     field_indices = gtir_domain.get_domain_indices(field_dims, field_origin)
     field_subset = dace_subsets.Range.from_indices(field_indices)
 
@@ -142,51 +161,36 @@ def _create_scan_field_operator_impl(
     )
     field_node = ctx.state.add_access(field_name)
 
-    # Now connect the final output node with the output of the nested SDFG.
+    # Now connect the output connector on the nested SDFG with the result field
+    #  outside the scan map scope. For 1D domain, containing only the column dimension,
+    #  there is no map scope, since the map range is only on the horizontal domain.
     #  Up to now the nested SDFG is writing into a transient data container that
     #  has the size to hold one column. The function below, that does the connection,
-    #  will remove that data container.
+    #  will remove that transient and write directly to the result field.
     inner_map_output_temporary_removed = output_edge.connect(map_exit, field_node, field_subset)
-    assert ctx.state.in_degree(field_node) == 1
+    if not inner_map_output_temporary_removed:
+        raise ValueError("The scan nested SDFG is expected to write directly to the result field.")
 
+    assert ctx.state.in_degree(field_node) == 1
     field_node_path = ctx.state.memlet_path(next(iter(ctx.state.in_edges(field_node))))
     assert field_node_path[-1].dst is field_node
 
-    if inner_map_output_temporary_removed:
-        # The original output of the nested SDFG, the one that would be inside the Map,
-        #  has been deleted and the nested SDFG writes directly into the output.
-        #  In this case we have to adapt the stride of the array inside the nested SDFG.
-        nsdfg_scan = field_node_path[0].src
-        assert isinstance(nsdfg_scan, dace.nodes.NestedSDFG)
-        inner_output_name = field_node_path[0].src_conn
-        inner_output_desc = nsdfg_scan.sdfg.arrays[inner_output_name]
-        assert len(inner_output_desc.shape) == 1
+    # The temporary node which the nested SDFG was writing to has been deleted,
+    #  and the nested SDFG will write directly to the result field. Thus, we have
+    #  to modify the stride of the scan column array inside the nested SDFG to match
+    #  the strides outside.
+    nsdfg_scan = field_node_path[0].src
+    assert isinstance(nsdfg_scan, dace.nodes.NestedSDFG)
+    inner_output_name = field_node_path[0].src_conn
+    inner_output_desc = nsdfg_scan.sdfg.arrays[inner_output_name]
+    assert len(inner_output_desc.shape) == 1
 
-        outside_output_stride = field_desc.strides[scan_dim_index]
-        assert str(outside_output_stride).isdigit()
-        # The stride of the temporary array is constant, so we can just set it on the inside.
-        inner_output_desc.set_shape(
-            new_shape=inner_output_desc.shape, strides=(outside_output_stride,)
-        )
-    else:
-        # The AccessNode on the inside of the Map was not removed but remains there.
-        #  Thus we do not have to update the strides, we do however, make some checks.
-        in_map_temporary_output_field = field_node_path[0].src
-        assert in_map_temporary_output_field == output_edge.result.dc_node
-
-        assert ctx.state.in_degree(in_map_temporary_output_field) == 1
-        assert ctx.state.out_degree(in_map_temporary_output_field) == 1
-        inner_edge = next(iter(ctx.state.in_edges(in_map_temporary_output_field)))
-        nsdfg_scan = inner_edge.src
-        assert isinstance(nsdfg_scan, dace.nodes.NestedSDFG)
-
-        inner_output_name = inner_edge.src_conn
-        inner_output_desc = nsdfg_scan.sdfg.arrays[inner_output_name]
-
-        assert len(inner_output_desc.shape) == 1
-        assert str(inner_output_desc.strides[0]).isdigit()
-        assert inner_output_desc.shape == dataflow_output_desc.shape
-        assert inner_output_desc.strides == dataflow_output_desc.strides
+    # The result field on the outside is a transient array, allocated inside this
+    # function, so we know that its stride is constant. We just need to set it on
+    # the inside array, and we do not need to map any stride symbol.
+    outside_output_stride = field_desc.strides[scan_dim_index]
+    assert str(outside_output_stride).isdigit()
+    inner_output_desc.set_shape(new_shape=inner_output_desc.shape, strides=(outside_output_stride,))
 
     return gtir_to_sdfg_types.FieldopData(
         field_node, ts.FieldType(field_dims, output_edge.result.gt_dtype), tuple(field_origin)
@@ -195,12 +199,12 @@ def _create_scan_field_operator_impl(
 
 def _create_scan_field_operator(
     ctx: gtir_to_sdfg.SubgraphContext,
-    domain: gtir_domain.FieldopDomain,
+    field_domain: gtir_domain.FieldopDomain,
     node_type: ts.FieldType | ts.TupleType,
     sdfg_builder: gtir_to_sdfg.SDFGBuilder,
     input_edges: Iterable[gtir_dataflow.DataflowInputEdge],
-    output_tree: gtir_dataflow.DataflowOutputEdge
-    | tuple[gtir_dataflow.DataflowOutputEdge | tuple[Any, ...], ...],
+    output: MaybeNestedInTuple[gtir_dataflow.DataflowOutputEdge | None],
+    output_domain: infer_domain.DomainAccess,
 ) -> gtir_to_sdfg_types.FieldopResult:
     """
     Helper method to build the output of a field operator, which can consist of
@@ -212,12 +216,17 @@ def _create_scan_field_operator(
     by a loop region in a mapped nested SDFG.
 
     Refer to `gtir_to_sdfg_primitives._create_field_operator()` for the
-    description of function arguments and return values.
+    description of function arguments. Note that the return value is different,
+    because the scan field operator can return a tuple of fields, while a regular
+    field operator return a single field. The domain of the nested fields, in
+    a tuple, can be empty, in case the nested field is not used outside the scan.
+    In this case, the corresponding `output` edge will be None and this function
+    will also return None for the corresponding field inside the tree-like result.
     """
-    domain_dims, _, _ = gtir_domain.get_field_layout(domain, ctx.target_domain)
+    dims, _, _ = gtir_domain.get_field_layout(field_domain)
 
     # create a map scope to execute the `LoopRegion` over the horizontal domain
-    if len(domain_dims) == 1:
+    if len(dims) == 1:
         # We construct the scan field operator on the horizontal domain, while the
         # vertical dimension (the column axis) is computed by the loop region.
         # If the field operator computes only the column axis (a 1d scan field operator),
@@ -232,41 +241,37 @@ def _create_scan_field_operator(
             "fieldop",
             ctx.state,
             ndrange={
-                gtir_to_sdfg_utils.get_map_variable(
-                    domain_range.dim
-                ): f"{domain_range.start}:{domain_range.stop}"
-                for domain_range in domain
-                if not sdfg_builder.is_column_axis(domain_range.dim)
+                gtir_to_sdfg_utils.get_map_variable(r.dim): f"{r.start}:{r.stop}"
+                for r in field_domain
+                if not sdfg_builder.is_column_axis(r.dim)
             },
         )
-        assert (len(map_entry.params) + 1) == len(domain)
+        assert (len(map_entry.params) + 1) == len(field_domain)
 
     # here we setup the edges passing through the map entry node
     for edge in input_edges:
         edge.connect(map_entry)
 
-    if isinstance(node_type, ts.FieldType):
-        assert isinstance(output_tree, gtir_dataflow.DataflowOutputEdge)
-        return _create_scan_field_operator_impl(
-            ctx, sdfg_builder, domain, output_tree, node_type, map_exit
-        )
-    else:
-        # handle tuples of fields
-        # the symbol name 'x' in the call below is not used, we only need
-        # the tree structure of the `TupleType` definition to pass to `tree_map()`
-        output_symbol_tree = gtir_to_sdfg_utils.make_symbol_tree("x", node_type)
-        return gtx_utils.tree_map(
-            lambda output_edge, output_sym: (
-                _create_scan_field_operator_impl(
-                    ctx,
-                    sdfg_builder,
-                    domain,
-                    output_edge,
-                    output_sym.type,
-                    map_exit,
-                )
+    # Note that `output_symbol` below is not used, we only need the tree-like
+    # structure to get the type of each nested field in the `tree_map` visitor.
+    dummy_output_symbol = (
+        gtir_to_sdfg_utils.make_symbol_tree("__gtir_unused_dummy_var", node_type)
+        if isinstance(node_type, ts.TupleType)
+        else im.sym("__gtir_unused_dummy_var", node_type)
+    )
+
+    return gtx_utils.tree_map(
+        lambda edge, domain, sym: (
+            _create_scan_field_operator_impl(
+                ctx,
+                sdfg_builder,
+                edge,
+                domain,
+                sym.type,
+                map_exit,
             )
-        )(output_tree, output_symbol_tree)
+        )
+    )(output, output_domain, dummy_output_symbol)
 
 
 def _scan_input_name(input_name: str) -> str:
@@ -288,9 +293,9 @@ def _lower_lambda_to_nested_sdfg(
     lambda_node: gtir.Lambda,
     ctx: gtir_to_sdfg.SubgraphContext,
     sdfg_builder: gtir_to_sdfg.SDFGBuilder,
-    domain: gtir_domain.FieldopDomain,
+    field_domain: gtir_domain.FieldopDomain,
     init_data: gtir_to_sdfg_types.FieldopResult,
-    lambda_symbols: dict[str, ts.DataType],
+    lambda_params: Sequence[gtir.Sym],
     scan_forward: bool,
     scan_carry_symbol: gtir.Sym,
 ) -> tuple[gtir_to_sdfg.SubgraphContext, gtir_to_sdfg_types.FieldopResult]:
@@ -309,13 +314,13 @@ def _lower_lambda_to_nested_sdfg(
     mapped by the caller to the horizontal domain in the field operator context.
 
     Args:
-        lambda_node: The lambda representing the stencil expression on the horizontal level.
+        lambda_node: The lambda representing the stencil expression on the scan dimension.
         ctx: The SDFG context where the scan field operator is translated.
         sdfg_builder: The SDFG builder object to access the field operator context.
-        domain: The field operator domain, with all horizontal and vertical dimensions.
+        field_domain: The field operator domain, with all horizontal and vertical dimensions.
         init_data: The data produced in the field operator context that is used
             to initialize the scan carry value.
-        lambda_symbols: List of symbols used as parameters of the stencil expressions.
+        lambda_params: List of symbols used as parameters of the lambda expression.
         scan_forward: When True, the loop should range starting from the origin;
             when False, traverse towards origin.
         scan_carry_symbol: The symbol used in the stencil expression to carry the
@@ -343,7 +348,12 @@ def _lower_lambda_to_nested_sdfg(
     )
     # the lambda expression, i.e. body of the scan, will be created inside a nested SDFG.
     lambda_translator, lambda_ctx = sdfg_builder.setup_nested_context(
-        lambda_node, "scan", ctx, lambda_symbols, symbolic_inputs=set()
+        lambda_node,
+        "scan",
+        ctx,
+        lambda_params,
+        symbolic_inputs=set(),
+        capture_scope_symbols=False,
     )
 
     # We set `using_explicit_control_flow=True` because the vertical scan is lowered to a `LoopRegion`.
@@ -352,9 +362,7 @@ def _lower_lambda_to_nested_sdfg(
     lambda_ctx.sdfg.using_explicit_control_flow = True
 
     # use the vertical dimension in the domain as scan dimension
-    scan_domain = next(
-        domain_range for domain_range in domain if sdfg_builder.is_column_axis(domain_range.dim)
-    )
+    scan_domain = next(r for r in field_domain if sdfg_builder.is_column_axis(r.dim))
 
     # extract the scan loop range
     scan_loop_var = gtir_to_sdfg_utils.get_map_variable(scan_domain.dim)
@@ -377,6 +385,7 @@ def _lower_lambda_to_nested_sdfg(
     if isinstance(init_data, tuple):
         lambda_result_shape = gtx_utils.tree_map(get_scan_output_shape)(init_data)
     else:
+        assert init_data is not None
         lambda_result_shape = get_scan_output_shape(init_data)
 
     # Create the body of the initialization state
@@ -433,14 +442,12 @@ def _lower_lambda_to_nested_sdfg(
     # The 'update' state writes the value computed by the stencil into the scan carry variable,
     # in order to make it available to the next vertical level.
     compute_state = scan_loop.add_state("scan_compute")
-    compute_ctx = gtir_to_sdfg.SubgraphContext(
-        lambda_ctx.sdfg, compute_state, lambda_ctx.target_domain
-    )
+    compute_ctx = gtir_to_sdfg.SubgraphContext(lambda_ctx.sdfg, compute_state)
     update_state = scan_loop.add_state_after(compute_state, "scan_update")
 
     # inside the 'compute' state, visit the list of arguments to be passed to the stencil
     stencil_args = [
-        _parse_scan_fieldop_arg(im.ref(p.id), compute_ctx, lambda_translator, domain)
+        _parse_scan_fieldop_arg(im.ref(p.id), compute_ctx, lambda_translator, field_domain)
         for p in lambda_node.params
     ]
     # stil inside the 'compute' state, generate the dataflow representing the stencil
@@ -475,7 +482,10 @@ def _lower_lambda_to_nested_sdfg(
         # `sym` represents the global output data, that is the nested-SDFG output connector
         scan_carry_data = str(scan_carry_sym.id)
         output = _scan_output_name(scan_carry_data)
-        lambda_ctx.sdfg.add_array(output, scan_output_shape, scan_result_desc.dtype)
+        # Note that we set `transient=True` because the lowering expects the dataflow
+        # of nested SDDFG to write to some internal temporary nodes. These data elements
+        # should be turned into globals by the caller and handled as output connections.
+        lambda_ctx.sdfg.add_array(output, scan_output_shape, scan_result_desc.dtype, transient=True)
         output_node = compute_state.add_access(output)
 
         # in the 'compute' state, we write the current vertical level data to the output field
@@ -501,15 +511,9 @@ def _lower_lambda_to_nested_sdfg(
 
     # write the stencil result (value on one vertical level) into a 1D field
     # with full vertical shape representing one column
-    if isinstance(scan_carry_input, tuple):
-        assert isinstance(lambda_result_shape, tuple)
-        lambda_output = gtx_utils.tree_map(connect_scan_output)(
-            lambda_result, lambda_result_shape, scan_carry_input
-        )
-    else:
-        assert isinstance(lambda_result[0], gtir_dataflow.DataflowOutputEdge)
-        assert isinstance(lambda_result_shape, list)
-        lambda_output = connect_scan_output(lambda_result[0], lambda_result_shape, scan_carry_input)
+    lambda_output = gtx_utils.tree_map(connect_scan_output)(
+        lambda_result, lambda_result_shape, scan_carry_input
+    )
 
     # in case tuples are passed as argument, isolated access nodes might be left in the state,
     # because not all tuple fields are necessarily accessed inside the lambda scope
@@ -527,39 +531,40 @@ def _lower_lambda_to_nested_sdfg(
     return lambda_ctx, lambda_output
 
 
-def _connect_nested_sdfg_output_to_temporaries(
+def _handle_dataflow_result_of_nested_sdfg(
+    nsdfg_node: dace.nodes.NestedSDFG,
     inner_ctx: gtir_to_sdfg.SubgraphContext,
     outer_ctx: gtir_to_sdfg.SubgraphContext,
-    nsdfg_node: dace.nodes.NestedSDFG,
     inner_data: gtir_to_sdfg_types.FieldopData,
-) -> gtir_dataflow.DataflowOutputEdge:
-    """
-    Helper function to create the edges to write output data from the nested SDFG
-    to temporary arrays in the parent SDFG, denoted as outer context.
-
-    Args:
-        inner_ctx: The inner SDFG context, where the scan `LoopRegion` is translated.
-        outer_ctx: The outer SDFG context, where the field operator is translated.
-        nsdfg_node: The nested SDFG node in the outer context.
-        inner_data: The data produced by the scan `LoopRegion` in the inner context.
-
-    Returns:
-        An object representing the output data connection of this field operator.
-    """
+    field_domain: infer_domain.NonTupleDomainAccess,
+) -> gtir_dataflow.DataflowOutputEdge | None:
     assert isinstance(inner_data.gt_type, ts.FieldType)
     inner_dataname = inner_data.dc_node.data
     inner_desc = inner_ctx.sdfg.data(inner_dataname)
-    outer_dataname, outer_desc = outer_ctx.sdfg.add_temp_transient_like(inner_desc)
-    outer_node = outer_ctx.state.add_access(outer_dataname)
-    outer_ctx.state.add_edge(
-        nsdfg_node,
-        inner_dataname,
-        outer_node,
-        None,
-        dace.Memlet.from_array(outer_dataname, outer_desc),
-    )
-    output_expr = gtir_dataflow.ValueExpr(outer_node, inner_data.gt_type.dtype)
-    return gtir_dataflow.DataflowOutputEdge(outer_ctx.state, output_expr)
+    assert inner_desc.transient
+
+    if isinstance(field_domain, domain_utils.SymbolicDomain):
+        # The field is used outside the nested SDFG, therefore it needs to be copied
+        # to a temporary array in the parent SDFG (outer context).
+        inner_desc.transient = False
+        outer_dataname, outer_desc = outer_ctx.sdfg.add_temp_transient_like(inner_desc)
+        outer_node = outer_ctx.state.add_access(outer_dataname)
+        outer_ctx.state.add_edge(
+            nsdfg_node,
+            inner_dataname,
+            outer_node,
+            None,
+            dace.Memlet.from_array(outer_dataname, outer_desc),
+        )
+        output_expr = gtir_dataflow.ValueExpr(outer_node, inner_data.gt_type.dtype)
+        return gtir_dataflow.DataflowOutputEdge(outer_ctx.state, output_expr)
+    else:
+        # The field is not used outside the nested SDFG. It is likely just storage
+        # for some internal state, accessed during column scan, and can be turned
+        # into a transient array inside the nested SDFG.
+        assert field_domain == infer_domain.DomainAccessDescriptor.NEVER
+        nsdfg_node.out_connectors.pop(inner_dataname)
+        return None
 
 
 def translate_scan(
@@ -585,11 +590,14 @@ def translate_scan(
 
     fun_node = node.fun
     assert len(fun_node.args) == 2
-    scan_expr, domain_expr = fun_node.args
+    scan_expr, scan_domain_expr = fun_node.args
     assert cpm.is_call_to(scan_expr, "scan")
 
     # parse the domain of the scan field operator
-    domain = gtir_domain.extract_domain(domain_expr)
+    assert isinstance(scan_domain_expr.type, ts.DomainType)
+    field_domain = gtir_domain.get_field_domain(
+        domain_utils.SymbolicDomain.from_expr(scan_domain_expr)
+    )
 
     # parse scan parameters
     assert len(scan_expr.args) == 3
@@ -597,7 +605,9 @@ def translate_scan(
     assert isinstance(stencil_expr, gtir.Lambda)
 
     # params[0]: the lambda parameter to propagate the scan carry on the vertical dimension
-    scan_carry = str(stencil_expr.params[0].id)
+    scan_carry = stencil_expr.params[0].id
+    scan_carry_type = stencil_expr.params[0].type
+    assert isinstance(scan_carry_type, ts.DataType)
 
     # params[1]: boolean flag for forward/backward scan
     assert isinstance(scan_expr.args[1], gtir.Literal) and ti.is_logical(scan_expr.args[1].type)
@@ -607,31 +617,25 @@ def translate_scan(
     init_expr = scan_expr.args[2]
     # visit the initialization value of the scan expression
     init_data = sdfg_builder.visit(init_expr, ctx=ctx)
-    # extract type definition of the scan carry
-    scan_carry_type = (
-        init_data.gt_type
-        if isinstance(init_data, gtir_to_sdfg_types.FieldopData)
-        else gtir_to_sdfg_types.get_tuple_type(init_data)
-    )
 
-    # define the set of symbols available in the lambda context, which consists of
-    # the carry argument and all lambda function arguments
-    lambda_arg_types = [scan_carry_type] + [
+    # define the symbols passed as parameter to the lambda expression, which consists
+    # of the carry argument and all lambda function arguments
+    lambda_arg_types: list[ts.DataType] = [scan_carry_type] + [
         arg.type for arg in node.args if isinstance(arg.type, ts.DataType)
     ]
-    lambda_symbols = {
-        str(p.id): arg_type
+    lambda_params = [
+        im.sym(p.id, arg_type)
         for p, arg_type in zip(stencil_expr.params, lambda_arg_types, strict=True)
-    }
+    ]
 
     # lower the scan stencil expression in a separate SDFG context
     lambda_ctx, lambda_output = _lower_lambda_to_nested_sdfg(
         stencil_expr,
         ctx,
         sdfg_builder,
-        domain,
+        field_domain,
         init_data,
-        lambda_symbols,
+        lambda_params,
         scan_forward,
         im.sym(scan_carry, scan_carry_type),
     )
@@ -643,59 +647,60 @@ def translate_scan(
     lambda_args_mapping = [
         (im.sym(_scan_input_name(scan_carry), scan_carry_type), init_data),
     ] + [
-        (im.sym(param.id, arg.gt_type), arg)
-        for param, arg in zip(stencil_expr.params[1:], lambda_args, strict=True)
+        (gt_symbol, arg)
+        for gt_symbol, arg in zip(stencil_expr.params[1:], lambda_args, strict=True)
+        if arg is not None
     ]
 
-    lambda_arg_nodes = dict(
-        itertools.chain(
-            *[gtir_to_sdfg_types.flatten_tuples(psym.id, arg) for psym, arg in lambda_args_mapping]
-        )
+    symbolic_args: dict[str, gtir_to_sdfg_types.SymbolicData] = {}
+    lambda_arg_nodes: dict[str, gtir_to_sdfg_types.FieldopData] = {}
+    for gtsym, arg in lambda_args_mapping:
+        gtsym_id = str(gtsym.id)
+        if isinstance(arg, gtir_to_sdfg_types.SymbolicData):
+            symbolic_args[gtsym_id] = arg
+        elif arg is not None:
+            lambda_arg_nodes |= {
+                str(nested_gtsym.id): nested_arg
+                for nested_gtsym, nested_arg in gtir_to_sdfg_types.flatten_tuple(gtsym, arg)
+                if nested_arg is not None
+            }
+
+    nsdfg_node, input_memlets = sdfg_builder.add_nested_sdfg(
+        stencil_expr, lambda_ctx, ctx, symbolic_args, lambda_arg_nodes, lambda_output
     )
 
-    # parse the dataflow output symbols
-    if isinstance(scan_carry_type, ts.TupleType):
-        lambda_flat_outs = {
-            str(sym.id): sym.type
-            for sym in gtir_to_sdfg_utils.flatten_tuple_fields(
-                _scan_output_name(scan_carry), scan_carry_type
-            )
-        }
-    else:
-        lambda_flat_outs = {_scan_output_name(scan_carry): scan_carry_type}
+    # Block the inlining of NestedSDFG containing a scan.
+    #  We do this to avoid a bug in DaCe simplify transformations, see
+    #  [issue#2182](https://github.com/spcl/dace/issues/2182) for more. Before the bug
+    #  was hidden but once we started running `TrivialTaskletElimination` it was no
+    #  longer the case. The solution is to block the inlining and keep scans localized
+    #  inside their NestedSDFG.
+    # NOTE: Currently there is no transformation that takes advantages in any way
+    #   of a scan and they are mostly inside Maps anyway, except of unit tests,
+    #   where inlining is not possible anyway.
+    nsdfg_node.no_inline = True
 
-    # build the mapping of symbols from nested SDFG to field operator context
-    nsdfg_symbols_mapping = {str(sym): sym for sym in lambda_ctx.sdfg.free_symbols}
-    for psym, arg in lambda_args_mapping:
-        nsdfg_symbols_mapping |= gtir_to_sdfg_utils.get_arg_symbol_mapping(psym.id, arg, ctx.sdfg)
-
-    # the scan nested SDFG is ready: it is instantiated in the field operator context
-    # where the map scope over the horizontal domain lives
-    nsdfg_node = ctx.state.add_nested_sdfg(
-        lambda_ctx.sdfg,
-        inputs=set(lambda_arg_nodes.keys()),
-        outputs=set(lambda_flat_outs.keys()),
-        symbol_mapping=nsdfg_symbols_mapping,
-    )
-
-    lambda_input_edges = []
-    for input_connector, outer_arg in lambda_arg_nodes.items():
-        arg_desc = outer_arg.dc_node.desc(ctx.sdfg)
-        input_subset = dace_subsets.Range.from_array(arg_desc)
+    input_edges = []
+    for input_connector, memlet in input_memlets.items():
+        src_node = lambda_arg_nodes[input_connector]
         input_edge = gtir_dataflow.MemletInputEdge(
-            ctx.state, outer_arg.dc_node, input_subset, nsdfg_node, input_connector
+            ctx.state, src_node.dc_node, memlet.src_subset, nsdfg_node, input_connector
         )
-        lambda_input_edges.append(input_edge)
+        input_edges.append(input_edge)
 
     # for output connections, we create temporary arrays that contain the computation
     # results of a column slice for each point in the horizontal domain
-    lambda_output_tree = gtx_utils.tree_map(
-        lambda lambda_output_data: _connect_nested_sdfg_output_to_temporaries(
-            lambda_ctx, ctx, nsdfg_node, lambda_output_data
+    output_tree = gtx_utils.tree_map(
+        lambda output_data, output_domain: _handle_dataflow_result_of_nested_sdfg(
+            nsdfg_node=nsdfg_node,
+            inner_ctx=lambda_ctx,
+            outer_ctx=ctx,
+            inner_data=output_data,
+            field_domain=output_domain,
         )
-    )(lambda_output)
+    )(lambda_output, node.annex.domain)
 
     # we call a helper method to create a map scope that will compute the entire field
     return _create_scan_field_operator(
-        ctx, domain, node.type, sdfg_builder, lambda_input_edges, lambda_output_tree
+        ctx, field_domain, node.type, sdfg_builder, input_edges, output_tree, node.annex.domain
     )
