@@ -14,7 +14,7 @@ that explains the general structure and requirements on the SDFGs.
 
 from . import constants, splitting_tools
 from .auto_optimize import GT4PyAutoOptHook, GT4PyAutoOptHookFun, gt_auto_optimize
-from .dead_dataflow_elimination import gt_eliminate_dead_dataflow
+from .dead_dataflow_elimination import gt_eliminate_dead_dataflow, gt_remove_map
 from .gpu_utils import (
     GPUSetBlockSize,
     gt_gpu_transform_non_standard_memlet,
@@ -38,13 +38,16 @@ from .map_fusion_extended import (
 )
 from .map_orderer import MapIterationOrder, gt_set_iteration_order
 from .map_promoter import MapPromoter, MapPromotionCallBack
+from .map_splitter import MapSplitter
+from .map_to_copy import MapToCopy
 from .move_dataflow_into_if_body import MoveDataflowIntoIfBody
 from .multi_state_global_self_copy_elimination import (
     MultiStateGlobalSelfCopyElimination,
     MultiStateGlobalSelfCopyElimination2,
     gt_multi_state_global_self_copy_elimination,
 )
-from .redundant_array_removers import CopyChainRemover, gt_remove_copy_chain
+from .redundant_array_removers import CopyChainRemover, DoubleWriteRemover, gt_remove_copy_chain
+from .remove_access_node_copies import RemoveAccessNodeCopies
 from .remove_views import RemovePointwiseViews
 from .simplify import (
     GT4PyMapBufferElimination,
@@ -62,17 +65,18 @@ from .split_access_nodes import SplitAccessNode, gt_split_access_nodes
 from .split_memlet import SplitConsumerMemlet
 from .state_fusion import GT4PyStateFusion
 from .strides import (
-    gt_change_transient_strides,
+    gt_change_strides,
     gt_map_strides_to_dst_nested_sdfg,
     gt_map_strides_to_src_nested_sdfg,
     gt_propagate_strides_from_access_node,
     gt_propagate_strides_of,
 )
-from .utils import gt_find_constant_arguments, gt_make_transients_persistent
+from .utils import gt_make_transients_persistent
 
 
 __all__ = [
     "CopyChainRemover",
+    "DoubleWriteRemover",
     "GPUSetBlockSize",
     "GT4PyAutoOptHook",
     "GT4PyAutoOptHookFun",
@@ -87,9 +91,12 @@ __all__ = [
     "MapIterationOrder",
     "MapPromoter",
     "MapPromotionCallBack",
+    "MapSplitter",
+    "MapToCopy",
     "MoveDataflowIntoIfBody",
     "MultiStateGlobalSelfCopyElimination",
     "MultiStateGlobalSelfCopyElimination2",
+    "RemoveAccessNodeCopies",
     "RemovePointwiseViews",
     "SingleStateGlobalDirectSelfCopyElimination",
     "SingleStateGlobalSelfCopyElimination",
@@ -99,10 +106,9 @@ __all__ = [
     "VerticalMapSplitCallback",
     "constants",
     "gt_auto_optimize",
-    "gt_change_transient_strides",
+    "gt_change_strides",
     "gt_create_local_double_buffering",
     "gt_eliminate_dead_dataflow",
-    "gt_find_constant_arguments",
     "gt_gpu_transform_non_standard_memlet",
     "gt_gpu_transformation",
     "gt_horizontal_map_split_fusion",
@@ -115,6 +121,7 @@ __all__ = [
     "gt_propagate_strides_of",
     "gt_reduce_distributed_buffering",
     "gt_remove_copy_chain",
+    "gt_remove_map",
     "gt_set_gpu_blocksize",
     "gt_set_iteration_order",
     "gt_simplify",
