@@ -14,7 +14,7 @@ from typing import Any, Callable, Optional, Sequence, TypeAlias, Union
 
 import dace
 from dace import data as dace_data
-from dace.sdfg import nodes as dace_nodes, propagation as dace_propagation
+from dace.sdfg import nodes as dace_nodes, propagation as dace_propagation, utils as dace_sdutils
 from dace.transformation.auto import auto_optimize as dace_aoptimize
 from dace.transformation.passes import analysis as dace_analysis
 
@@ -332,6 +332,32 @@ def gt_auto_optimize(
         if gpu_block_size_3d is not None:
             gpu_block_size_spec["block_size_3d"] = gpu_block_size_3d
 
+        # We now turn the demoted fields back into globals if possible, for ABI compatibility.
+        #  We have to do it here, because we need to know later if something is a transient
+        #  or not.
+        for demoted_field, original_field_desc in original_demoted_descriptors.items():
+            if demoted_field not in sdfg.arrays:
+                # The demoted field is not inside the SDFG anymore, just insert it.
+                sdfg.add_datadesc(demoted_field, original_field_desc)
+
+            elif original_field_desc.is_equivalent(sdfg.arrays[demoted_field]) and all(
+                (ostride == cstride) == True  # noqa: E712 [true-false-comparison]  # SymPy comparison
+                for ostride, cstride in zip(
+                    sdfg.arrays[demoted_field].strides, original_field_desc.strides
+                )
+            ):
+                # The demoted field is still inside the SDFG, we only add it back if it
+                #  still has the same layout as before.
+                # NOTE: Technically we could ignore the strides, the problem are nested
+                #   SDFG, where they might be mapped into.
+                sdfg.arrays[demoted_field].transient = False
+
+            else:
+                warnings.warn(
+                    f"Could not restore the demoted field '{demoted_field}' back to a global.",
+                    stacklevel=0,
+                )
+
         sdfg = _gt_auto_configure_maps_and_strides(
             sdfg=sdfg,
             gpu=gpu,
@@ -358,33 +384,14 @@ def gt_auto_optimize(
             validate_all=validate_all,
         )
 
+        # Canonicalize the SDFG. This ensures that the code generator will see SDFGs
+        #  that conform to the historical expected version.
+        # NOTE: Currently needed to avoid an error in `AddThreadBlockMap`.
+        dace.sdfg.utils.canonicalize_memlet_trees(sdfg)
+
         # Set the implementation of the library nodes.
         dace_aoptimize.set_fast_implementations(sdfg, device)
 
-        # We now turn the demoted fields back into globals if possible, for ABI compatibility.
-        for demoted_field, original_field_desc in original_demoted_descriptors.items():
-            if demoted_field not in sdfg.arrays:
-                # The demoted field is not inside the SDFG anymore, just insert it.
-                sdfg.add_datadesc(demoted_field, original_field_desc)
-
-            elif original_field_desc.is_equivalent(sdfg.arrays[demoted_field]) and all(
-                (ostride == cstride) == True  # noqa: E712 [true-false-comparison]  # SymPy comparison
-                for ostride, cstride in zip(
-                    sdfg.arrays[demoted_field].strides, original_field_desc.strides
-                )
-            ):
-                # The demoted field is still inside the SDFG, we only add it back if it
-                #  still has the same layout as before.
-                # NOTE: Technically we could ignore the strides, the problem are nested
-                #   SDFG, where they might be mapped into.
-                sdfg.arrays[demoted_field].transient = False
-
-            else:
-                warnings.warn(
-                    f"Could not restore the demoted field '{demoted_field}' back to a global.",
-                    stacklevel=0,
-                )
-
         if validate:
             sdfg.validate()
 
@@ -434,6 +441,7 @@ def _gt_auto_process_top_level_maps(
     #  We use the hash instead of the return values of the transformation, because
     #  computing the hash invalidates some caches that are not properly updated in Dace.
     # TODO(phimuell): Remove this hack as soon as DaCe is fixed.
+    # TODO(phimuell): Maybe switch to `reset_cfg_list()`?
     sdfg_hash = sdfg.hash_sdfg()
 
     if GT4PyAutoOptHook.TopLevelDataFlowPre in optimization_hooks:
@@ -458,39 +466,26 @@ def _gt_auto_process_top_level_maps(
         #   has been solved.
         vertical_map_fusion._single_use_data = single_use_data
 
-        horizontal_map_fusion = gtx_transformations.MapFusionHorizontal(
-            only_toplevel_maps=True,
-            consolidate_edges_only_if_not_extending=True,
-            # TODO(phimuell): Should we enable this flag?
-            only_if_common_ancestor=False,
-            check_fusion_callback=optimization_hooks.get(  # type: ignore[arg-type]
-                GT4PyAutoOptHook.TopLevelDataFlowMapFusionHorizontalCallBack, None
-            ),
-        )
-
-        # NOTE: Since horizontal Map fusion matches _any_ two Maps, running it on large
-        #  SDFGs is expensive. Thus we run vertical Map fusion first to reduce the search
-        #  space.
         sdfg.apply_transformations_repeated(
             vertical_map_fusion,
             validate=False,
             validate_all=validate_all,
         )
+
+        # Try to remove some unnecessary copies.
         sdfg.apply_transformations_repeated(
-            [horizontal_map_fusion, vertical_map_fusion],
+            gtx_transformations.DoubleWriteRemover(
+                single_use_data=single_use_data,
+            ),
             validate=False,
             validate_all=validate_all,
         )
 
-        # We have to call it here, such that some other transformations, most
-        #  importantly the split transformations run.
-        #  We allow promotion of vertical (which is beneficial for Nabla4 type
-        #  stencils and if `LoopBlocking` is enabled. However, we also allow
-        #  horizontal promotion, this is an empirical choice. If the computation
-        #  on the horizontal are very complicated, it might degrade performance
-        #  but such cases should be rare.
-        # TODO(phimuell): Add a criteria to decide if we should promote or not.
-        # TODO(phimuell): Think about relocating this before horizontal Map fusion.
+        # Promote Maps. This will remove transients between 1D and 2D Maps, at the
+        #  cost of more data loads from memory. Empirical observations have shown
+        #  that this is beneficial; especially for Nabla4-type kernel in conjunction
+        #  with `LoopBlocking`.
+        # NOTE: We have to promote before we do horizontal Map fusion.
         sdfg.apply_transformations_repeated(
             gtx_transformations.MapPromoter(
                 only_toplevel_maps=True,
@@ -507,6 +502,31 @@ def _gt_auto_process_top_level_maps(
             validate_all=validate_all,
         )
 
+        # Now run horizontal Map fusion, also run vertical Map fusion, because it
+        #  might have become possible.
+        # TODO(phimuell): Think of moving horizontal Map fusion after the splitting.
+        horizontal_map_fusion = gtx_transformations.MapFusionHorizontal(
+            only_toplevel_maps=True,
+            consolidate_edges_only_if_not_extending=True,
+            check_fusion_callback=optimization_hooks.get(  # type: ignore[arg-type]
+                GT4PyAutoOptHook.TopLevelDataFlowMapFusionHorizontalCallBack, None
+            ),
+            # NOTE: Setting this argument to `False` has the side effect that Maps
+            #   without any connection to each other are fused together and by this
+            #   creating dependencies between otherwise unrelated Maps. However,
+            #   also has the side effect of integrating Maps generated in
+            #   `broadcast+concat_where` situations into other Maps. We should handle
+            #   these cases through the splitting transformations.
+            # TODO(phimuell): Update the other transformations such that we can
+            #   set the falg to `True`.
+            only_if_common_ancestor=False,
+        )
+        sdfg.apply_transformations_repeated(
+            [horizontal_map_fusion, vertical_map_fusion],
+            validate=False,
+            validate_all=validate_all,
+        )
+
         # Now do some cleanup task, that may enable further fusion opportunities.
         #  Note for performance reasons simplify is deferred.
         gtx_transformations.gt_reduce_distributed_buffering(
@@ -514,13 +534,23 @@ def _gt_auto_process_top_level_maps(
         )
 
         if not disable_splitting:
-            # TODO(phimuell): Find out how to skip the propagation and integrating it
-            #   into the split transformation.
+            # TODO(phimuell): Implement a data cleaner.
+            dace_sdutils.canonicalize_memlet_trees(sdfg)
+            dace_propagation.propagate_memlets_sdfg(sdfg)
             sdfg.apply_transformations_repeated(
-                gtx_transformations.SplitConsumerMemlet(single_use_data=single_use_data),
+                [
+                    gtx_transformations.MapSplitter(
+                        single_use_data=single_use_data,
+                        remove_dead_dataflow=True,
+                    ),
+                    gtx_transformations.SplitConsumerMemlet(single_use_data=single_use_data),
+                ],
                 validate=False,
                 validate_all=validate_all,
             )
+            # TODO(phimuell): Find out how to skip the propagation and integrating it
+            #   into the split transformation.
+            dace_sdutils.canonicalize_memlet_trees(sdfg)
             dace_propagation.propagate_memlets_sdfg(sdfg)
 
             sdfg.apply_transformations_repeated(
@@ -732,7 +762,7 @@ def _gt_auto_configure_maps_and_strides(
     #   bad thing is to set the strides of the transients here. The main downside
     #   is that this and the `_gt_auto_post_processing()` function has these weird
     #   names.
-    gtx_transformations.gt_change_transient_strides(sdfg, gpu=gpu)
+    gtx_transformations.gt_change_strides(sdfg, gpu=gpu)
 
     if gpu:
         # TODO(phimuell): The GPU function might modify the map iteration order.
