@@ -16,7 +16,12 @@ from dace import (
     subsets as dace_subsets,
     transformation as dace_transformation,
 )
-from dace.sdfg import graph as dace_graph, nodes as dace_nodes, propagation as dace_propagation
+from dace.sdfg import (
+    graph as dace_graph,
+    nodes as dace_nodes,
+    propagation as dace_propagation,
+    utils as dace_sdutils,
+)
 from dace.transformation.dataflow import map_fusion_helper as dace_mfhelper
 from dace.transformation.passes import analysis as dace_analysis
 
@@ -347,8 +352,10 @@ class SplitMapRange(dace_transformation.SingleStateTransformation):
         #  by assumption inside the same Map.
         if containing_scope_or_none is None:
             for new_map_entry in new_map_entries_from_first + new_map_entries_from_second:
+                dace_sdutils.canonicalize_memlet_trees_for_map(graph, new_map_entry)
                 dace_propagation.propagate_memlets_map_scope(sdfg, graph, new_map_entry)
         else:
+            dace_sdutils.canonicalize_memlet_trees_for_map(graph, containing_scope_or_none)
             dace_propagation.propagate_memlets_map_scope(sdfg, graph, containing_scope_or_none)
 
         # Workaround to ensure that some cache in DaCe has been cleared.
