@@ -11,6 +11,7 @@
 import collections
 import copy
 import uuid
+import warnings
 from typing import Any, Iterable, Optional, TypeAlias
 
 import dace
@@ -21,7 +22,7 @@ from dace import (
     transformation as dace_transformation,
 )
 from dace.cli import progress as dace_cliprogress
-from dace.sdfg import nodes as dace_nodes
+from dace.sdfg import nodes as dace_nodes, utils as dace_sdutils
 from dace.transformation import (
     dataflow as dace_dataflow,
     interstate as dace_interstate,
@@ -62,6 +63,10 @@ def gt_simplify(
         `concat_where` built-in function.
     - `GT4PyDeadDataflowElimination`: Run `gt_eliminate_dead_dataflow()` on the SDFG,
         which removes more dead dataflow than the native DaCe version.
+    - `MapToCopy`: Called to remove some slices.
+    - `canonicalize_memlet_trees()`: A free function that will canonicalize all Memlets.
+        It is run after the inlining and can not be disabled.
+    - `TrivialTaskletElimination`: Removing trivial copies.
 
     Furthermore, by default, or if `None` is passed for `skip` the passes listed in
     `GT_SIMPLIFY_DEFAULT_SKIP_SET` will be skipped.
@@ -109,6 +114,14 @@ def gt_simplify(
                 result = result or {}
                 result.update(inline_res)
 
+        # Ensure that we have canonical Memelts.
+        canoncialize_memlet_result = dace_sdutils.canonicalize_memlet_trees(sdfg)
+        if canoncialize_memlet_result:
+            result = result or {}
+            if "canonicalize_memlet_trees" not in result:
+                result["canonicalize_memlet_trees"] = 0
+            result["canonicalize_memlet_trees"] += canoncialize_memlet_result
+
         simplify_res = dace_passes.SimplifyPass(
             validate=False,
             validate_all=validate_all,
@@ -138,6 +151,23 @@ def gt_simplify(
                     result["FuseStates"] = 0
                 result["FuseStates"] += fuse_state_res
 
+        if "MapToCopy" not in skip:
+            find_single_use_data = dace_transformation.passes.analysis.FindSingleUseData()
+            single_use_data = find_single_use_data.apply_pass(sdfg, None)
+            removed_maps = sdfg.apply_transformations_once_everywhere(
+                gtx_transformations.MapToCopy(
+                    single_use_data=single_use_data,
+                ),
+                validate=False,
+                validate_all=validate_all,
+            )
+            if removed_maps:
+                at_least_one_xtrans_run = True
+                result = result or {}
+                if "MapToCopy" not in result:
+                    result["MapToCopy"] = 0
+                result["MapToCopy"] += removed_maps
+
         if "GT4PyDeadDataflowElimination" not in skip:
             eliminate_dead_dataflow_res = gtx_transformations.gt_eliminate_dead_dataflow(
                 sdfg=sdfg,
@@ -152,6 +182,19 @@ def gt_simplify(
                     result["GT4PyDeadDataflowElimination"] = 0
                 result["GT4PyDeadDataflowElimination"] += eliminate_dead_dataflow_res
 
+        if "TrivialTaskletElimination" not in skip:
+            eliminated_trivial_tasklets = sdfg.apply_transformations_once_everywhere(
+                dace.transformation.dataflow.TrivialTaskletElimination(),
+                validate=False,
+                validate_all=validate_all,
+            )
+            if eliminated_trivial_tasklets:
+                at_least_one_xtrans_run = True
+                result = result or {}
+                if "TrivialTaskletElimination" not in result:
+                    result["TrivialTaskletElimination"] = 0
+                result["TrivialTaskletElimination"] += eliminated_trivial_tasklets
+
         if "CopyChainRemover" not in skip:
             copy_chain_remover_result = gtx_transformations.gt_remove_copy_chain(
                 sdfg=sdfg,
@@ -283,6 +326,12 @@ def gt_inline_nested_sdfg(
         if multi_state_inliner.can_be_applied(parent_state, 0, parent_sdfg, permissive=permissive):
             multi_state_inliner.apply(parent_state, parent_sdfg)
             nb_inlines_total += 1
+            if nsdfg_node.label.startswith("scan_"):
+                # See `gtir_to_sdfg_scan.py::translate_scan()` for more information.
+                warnings.warn(
+                    f"Inlined '{nsdfg_node.label}' which might be a scan, this might leads to errors during simplification.",
+                    stacklevel=0,
+                )
 
     result: dict[str, int] = {}
     if nb_inlines_total != 0:
@@ -370,6 +419,8 @@ def gt_substitute_compiletime_symbols(
             validate=False,
             validate_all=validate_all,
         )
+    else:
+        dace_sdutils.canonicalize_memlet_trees(sdfg)
     dace.sdfg.propagation.propagate_memlets_sdfg(sdfg)
 
     if validate:
@@ -443,6 +494,7 @@ class DistributedBufferRelocator(dace_transformation.Pass):
             transient this is okay, as our rule guarantees this.
 
     Todo:
+        - Completely remove `temp_storage`, i.e. write directly into `dest_storage`.
         - Allow that `dest_storage` can also be transient.
         - Allow that `dest_storage` does not need to be a sink node, this is most
             likely most relevant if it is transient.
@@ -504,6 +556,8 @@ class DistributedBufferRelocator(dace_transformation.Pass):
             final_dest_name: str = wb_edge.dst.data
 
             for def_an, def_state in def_locations:
+                # TODO(phimuell): Do not create a copy from `temp_storage` to the newly
+                #   created `dest_storage`. Instead bypass `temp_storage` fully.
                 def_state.add_edge(
                     def_an,
                     wb_edge.src_conn,
