@@ -28,7 +28,7 @@ inside the NestedSDFG.
 """
 
 
-def gt_change_transient_strides(
+def gt_change_strides(
     sdfg: dace.SDFG,
     gpu: bool,
 ) -> dace.SDFG:
@@ -49,50 +49,45 @@ def gt_change_transient_strides(
 
     Note:
         Currently the function will not scan the access pattern. Instead it will
-        either use FORTRAN order for GPU or C order (which is assumed to be the
-        default, so it is a no ops).
+        either use FORTRAN order for GPU or C order. This function needs to be called
+        for both CPU and GPU to handle strides of memlets inside nested SDFGs.
 
     Todo:
         - Implement the estimation correctly.
     """
     # TODO(phimeull): Implement this function correctly.
 
-    # We assume that by default we have C order which is already correct,
-    #  so in this case we have a no ops
-    if not gpu:
-        return sdfg
-
     for nsdfg in sdfg.all_sdfgs_recursive():
-        _gt_change_transient_strides_non_recursive_impl(nsdfg)
+        _gt_change_strides_non_recursive_impl(nsdfg, gpu)
 
 
-def _gt_change_transient_strides_non_recursive_impl(
+def _gt_change_strides_non_recursive_impl(
     sdfg: dace.SDFG,
+    gpu: bool,
 ) -> None:
-    """Set optimal strides of all transients in the SDFG.
+    """Set optimal strides of all access nodes in the SDFG.
 
-    The function will look for all top level transients, see `_gt_find_toplevel_data_accesses()`
+    The function will look for all top level access node, see `_gt_find_toplevel_data_accesses()`
     and set their strides such that the access is optimal, see Note. The function
     will also run `gt_propagate_strides_of()` to propagate the strides into nested SDFGs.
 
-    This function should never be called directly but always through
-    `gt_change_transient_strides()`!
+    This function should never be called directly but always through `gt_change_strides()`!
 
     Note:
         Currently the function just reverses the strides of the data descriptor
-        it processes. Since DaCe generates `C` order by default this lead to
-        FORTRAN order, which is (for now) sufficient to optimize the memory
+        of transient access nodes it processes. Since DaCe generates `C` order by default
+        this lead to FORTRAN order, which is (for now) sufficient to optimize the memory
         layout to GPU.
 
     Todo:
         Make this function more intelligent to analyse the access pattern and then
         figuring out the best order.
     """
-    # NOTE: Processing the transient here is enough. If we are inside a
+    # NOTE: We have to process all access nodes (transient and globals). If we are inside a
     #   NestedSDFG then they were handled before on the level above us.
     top_level_transients_and_their_accesses = _gt_find_toplevel_data_accesses(
         sdfg=sdfg,
-        only_transients=True,
+        only_transients=False,
         only_arrays=True,
     )
     for top_level_transient, accesses in top_level_transients_and_their_accesses.items():
@@ -104,19 +99,20 @@ def _gt_change_transient_strides_non_recursive_impl(
             continue
 
         # We assume that everything is in C order initially, to get FORTRAN order
-        #  we simply have to reverse the order.
-        # TODO(phimuell): Improve this.
-        new_stride_order = list(range(ndim))
-        desc.set_strides_from_layout(*new_stride_order)
+        #  we simply have to reverse the order. This is necessary only for transient
+        #  access nodes because the non-transients come from outside and have their
+        #  own strides.
+        # TODO(phimuell): Set the stride based on the actual access pattern.
+        if desc.transient and gpu:
+            new_stride_order = list(range(ndim))
+            desc.set_strides_from_layout(*new_stride_order)
 
         # Now we have to propagate the changed strides. Because we already have
         #  collected all the AccessNodes we are using the
         #  `gt_propagate_strides_from_access_node()` function, but we have to
         #  create `processed_nsdfg` set already outside here.
-        #  Furthermore, the same comment as above applies here, we do not have to
-        #  propagate the non-transients, because they either come from outside,
-        #  or they were already handled in the levels above, where they were
-        #  defined and then propagated down.
+        #  While global access nodes in top level SDFGs should have the correct strides,
+        #  we need to propagate those strides into the nested SDFGs that use them.
         # TODO(phimuell): Updated the functions such that only one scan is needed.
         processed_nsdfgs: set[dace_nodes.NestedSDFG] = set()
         for state, access_node in accesses:
@@ -128,6 +124,10 @@ def _gt_change_transient_strides_non_recursive_impl(
                 ignore_symbol_mapping=True,
             )
 
+    # Now handle the views.
+    # TODO(phimuell): Remove once `gt_propagate_strides_from_access_node()` can handle views.
+    _gt_modify_strides_of_views_non_recursive(sdfg)
+
 
 def gt_propagate_strides_of(
     sdfg: dace.SDFG,
@@ -191,6 +191,10 @@ def gt_propagate_strides_from_access_node(
             of NestedSDFGs instead of manipulating the data descriptor.
         processed_nsdfgs: Set of NestedSDFG that were already processed and will be ignored.
             Only specify when you know what your are doing.
+
+    Todo:
+        Once this function supports the propagation to views `_gt_modify_strides_of_views()`
+        can be removed.
     """
     assert isinstance(state, dace.SDFGState)
 
@@ -612,15 +616,14 @@ def _gt_find_toplevel_data_accesses(
     scope and error is generated.
     By default the function will return transient and non transient data,
     however, if `only_transients` is `True` then only transient data will
-    be returned.
-    Furthermore, the function will ignore an access in the following cases:
-    - The AccessNode refers to data that is a register.
-    - The AccessNode refers to a View.
+    be returned. In addition by seeting `only_arrays` only arrays will be
+    returned.
 
     Args:
         sdfg: The SDFG to process.
         only_transients: If `True` only include transients.
         only_arrays: If `True`, defaults to `False`, only arrays are returned.
+        inlclude_views: Also returns Views.
 
     Returns:
         A `dict` that maps the name of a data container, to a list of tuples
@@ -682,3 +685,81 @@ def _gt_find_toplevel_data_accesses(
             # Now create the new entry in the list and record the AccessNode.
             top_level_data[data] = [(state, dnode)]
     return top_level_data
+
+
+def _gt_modify_strides_of_views_non_recursive(sdfg: dace.SDFG) -> None:
+    """The function determines the strides of Views.
+
+    The function should not be called directly, instead it is called by
+    `gt_change_strides()` directly if needed. The function will recursively
+    process the SDFG and modifies the strides.
+
+    Todo:
+        Once `gt_propagate_strides_from_access_node()` can handle View updates
+        evaluate if this function is still needed.
+    """
+    for state in sdfg.states():
+        scope_dict = state.scope_dict()
+        propagation_record: set[PropagatedStrideRecord] = set()
+        for view_node in state.data_nodes():
+            view_desc = view_node.desc(sdfg)
+            if not isinstance(view_desc, dace_data.View):
+                continue
+            viewed_node = gtx_transformations.utils.track_view(view_node, state, sdfg)
+            viewed_desc = viewed_node.desc(sdfg)
+
+            # We are not able to handle tower of Views
+            if isinstance(viewed_desc, dace_data.View):
+                raise NotImplementedError(
+                    f"Can not handle the view '{view_node.data}' that views view '{viewed_node.data}'"
+                )
+
+            # If both the View and the viewed node are not on the top level then do
+            #  nothing. Why? The answer is that this function is only called by
+            #  `_gt_change_strides_non_recursive_impl()` which only manipulates
+            #  the strides of transients at the top level and leaves the one inside
+            #  a Map alone. Thus there was no modification and no change.
+            if scope_dict[viewed_node] is not None and scope_dict[view_node] is not None:
+                continue
+
+            # If the viewed data is global data, then we do not modify the strides because
+            #  we assume that it was set correctly from the beginning and the viewed strides
+            #  have not changed.
+            # TODO(phimuell): Check interaction with the demote feature of auto optimizer.
+            if not viewed_desc.transient:
+                continue
+
+            # There is a special case if the View is a scalar or has a shape of `(1,)`
+            #  then strides are meaningless.
+            if isinstance(view_desc, dace_data.Scalar) or (
+                len(view_desc.shape) == 1 and ((view_desc.shape[0] == 1) == True)  # noqa: E712 [true-false-comparison]  # SymPy comparison
+            ):
+                continue
+
+            # If the dimensionality of the two data is different then we can not handle it.
+            #  This is probably the most difficult case to handle. However, instead of
+            #  handling the case, we should simply remove the View.
+            if len(view_desc.shape) != len(viewed_desc.shape):
+                raise NotImplementedError(
+                    f"Can not handle the change from {len(viewed_desc.shape)} ({viewed_node.data}) to {len(view_desc.shape)} ({view_node.data})."
+                )
+
+            # Even if they have the same dimensionality, we can not simply copy the strides.
+            #  Consider the case were `viewed_desc` is a 2D array with the following shape
+            #  `(N, M)`. `view_desc` is a vertical slice, but for some reasons has an
+            #  additional dummy dimension, i.e. its shape is either `(N, 1)` or `(1, N)`.
+            #  So copying the strides around is not gonna work.
+            if view_desc.shape != viewed_desc.shape:
+                raise NotImplementedError(
+                    f"Can not change from shape `{viewed_desc.shape}` ({viewed_node.data})to shape `{view_desc.shape}` ({view_node.data})."
+                )
+
+            # In case they have the same shape we are fine.
+            view_desc.strides = viewed_desc.strides
+            gt_propagate_strides_from_access_node(
+                sdfg=sdfg,
+                state=state,
+                outer_node=view_node,
+                ignore_symbol_mapping=True,
+                processed_nsdfgs=propagation_record,
+            )
