@@ -9,11 +9,14 @@
 from __future__ import annotations
 
 import re
-from typing import Final, Literal, Mapping, Optional, Union
+from typing import Final, Literal, Mapping, Union
 
 import dace
 
 from gt4py.next import common as gtx_common
+from gt4py.next.iterator import builtins as gtir_builtins
+from gt4py.next.iterator.ir_utils import ir_makers as im
+from gt4py.next.program_processors.runners.dace import gtir_python_codegen
 from gt4py.next.type_system import type_specifications as ts
 
 
@@ -21,14 +24,11 @@ from gt4py.next.type_system import type_specifications as ts
 CONNECTIVITY_INDENTIFIER_PREFIX: Final[str] = "gt_conn_"
 CONNECTIVITY_INDENTIFIER_RE: Final[re.Pattern] = re.compile(r"^gt_conn_(\S+)$")
 
-# regex for domain range symbols
-RANGE_SYMBOL_RE: Final[re.Pattern] = re.compile(r"^__(\S+)_(\S+)_range_[01]$")
+# regex for field size/stride symbol name
+FIELD_SYMBOL_RE: Final[re.Pattern] = re.compile(r"^__(\S+)_(\S+)_(size|stride)$")
 
-# regex for field stride symbols
-SIZE_SYMBOL_RE: Final[re.Pattern] = re.compile(r"^__(\S+)_size_\d+$")
-
-# regex for field stride symbols
-STRIDE_SYMBOL_RE: Final[re.Pattern] = re.compile(r"^__(\S+)_stride_\d+$")
+# element data type for field size/stride symbols
+FIELD_SYMBOL_DTYPE: Final[dace.typeclass] = getattr(dace, gtir_builtins.INTEGER_INDEX_BUILTIN)
 
 
 def as_dace_type(type_: ts.ScalarType) -> dace.typeclass:
@@ -58,7 +58,7 @@ def connectivity_identifier(name: str) -> str:
 
 
 def is_connectivity_identifier(
-    name: str, offset_provider_type: Optional[gtx_common.OffsetProviderType] = None
+    name: str, offset_provider_type: gtx_common.OffsetProviderType | None = None
 ) -> bool:
     m = CONNECTIVITY_INDENTIFIER_RE.match(name)
     if m is None:
@@ -71,52 +71,68 @@ def is_connectivity_identifier(
 
 
 def is_connectivity_symbol(name: str, offset_provider_type: gtx_common.OffsetProviderType) -> bool:
-    m = SIZE_SYMBOL_RE.match(name) or STRIDE_SYMBOL_RE.match(name)
-    if m is None:
+    if (m_symbol := FIELD_SYMBOL_RE.match(name)) is None:
         return False
-    m = CONNECTIVITY_INDENTIFIER_RE.match(m[1])
-    if m is None:
+    if (m := CONNECTIVITY_INDENTIFIER_RE.match(m_symbol[1])) is None:
         return False
     return gtx_common.has_offset(offset_provider_type, m[1])
 
 
-def field_symbol_name(field_name: str, axis: int, sym: Literal["size", "stride"]) -> str:
-    return f"__{field_name}_{sym}_{axis}"
+def _field_symbol(
+    field_name: str,
+    dim: gtx_common.Dimension,
+    sym: Literal["size", "stride"],
+    offset_provider_type: Mapping[str, gtx_common.NeighborConnectivityType] | None,
+) -> dace.symbol:
+    if (m := CONNECTIVITY_INDENTIFIER_RE.match(field_name)) is None:
+        name = f"__{field_name}_{dim.value}_{sym}"
+    else:  # a connectivity field
+        assert offset_provider_type is not None
+        assert m[1] in offset_provider_type
+        offset = m[1]
+        conn_type = offset_provider_type[offset]
+        if dim == conn_type.source_dim:
+            name = f"__{field_name}_source_{sym}"
+        elif dim == conn_type.neighbor_dim:
+            name = f"__{field_name}_neighbor_{sym}"
+        else:
+            raise ValueError(f"Unexpect dimension '{dim}' for '{offset}' connectivity.")
+    return dace.symbol(name, FIELD_SYMBOL_DTYPE)
 
 
-def field_size_symbol_name(field_name: str, axis: int) -> str:
-    return field_symbol_name(field_name, axis, "size")
+def field_size_symbol(
+    field_name: str,
+    dim: gtx_common.Dimension,
+    offset_provider_type: Mapping[str, gtx_common.NeighborConnectivityType],
+) -> dace.symbol:
+    return _field_symbol(field_name, dim, "size", offset_provider_type)
 
 
-def field_stride_symbol_name(field_name: str, axis: int) -> str:
-    return field_symbol_name(field_name, axis, "stride")
+def field_stride_symbol(
+    field_name: str,
+    dim: gtx_common.Dimension,
+    offset_provider_type: Mapping[str, gtx_common.NeighborConnectivityType] | None = None,
+) -> dace.symbol:
+    return _field_symbol(field_name, dim, "stride", offset_provider_type)
 
 
-def range_symbol(field_name: str, axis: str) -> str:
+def _range_symbol_name(field_name: str, axis: str) -> str:
     """Common part of the name for the range start/stop symbols."""
-    return f"__{field_name}_{axis}_range"
+    dim = gtx_common.Dimension(axis)
+    field_range = im.call("get_domain_range")(field_name, dim)
+    return gtir_python_codegen.get_source(field_range)
 
 
-def range_start_symbol(field_name: str, dim: gtx_common.Dimension) -> str:
+def range_start_symbol(field_name: str, dim: gtx_common.Dimension) -> dace.symbol:
     """Format name of the start symbol for domain range."""
-    return f"{range_symbol(field_name, dim.value)}_0"
+    name = f"{_range_symbol_name(field_name, dim.value)}_0"
+    return dace.symbol(name, FIELD_SYMBOL_DTYPE)
 
 
-def range_stop_symbol(field_name: str, dim: gtx_common.Dimension) -> str:
+def range_stop_symbol(field_name: str, dim: gtx_common.Dimension) -> dace.symbol:
     """Format name of the stop symbol for domain range."""
-    return f"{range_symbol(field_name, dim.value)}_1"
-
-
-def is_range_symbol(name: str) -> bool:
-    return RANGE_SYMBOL_RE.match(name) is not None
-
-
-def is_size_symbol(name: str) -> bool:
-    return SIZE_SYMBOL_RE.match(name) is not None
-
-
-def is_stride_symbol(name: str) -> bool:
-    return STRIDE_SYMBOL_RE.match(name) is not None
+    name = f"{_range_symbol_name(field_name, dim.value)}_1"
+    return dace.symbol(name, FIELD_SYMBOL_DTYPE)
 
 
 def filter_connectivity_types(
