@@ -10,6 +10,7 @@ from __future__ import annotations
 
 import dataclasses
 import os
+import warnings
 from collections.abc import Callable, MutableSequence, Sequence
 from typing import Any
 
@@ -36,6 +37,14 @@ class CompiledDaceProgram(stages.CompiledProgram):
         None,
     ]
 
+    # Processed argument vectors that are passed to `CompiledSDFG.fast_call()`. `None`
+    #  means that it has not been initialized, i.e. no call was ever performed.
+    #  - csdfg_argv: Arguments used for calling the actual compiled SDFG, will be updated.
+    #  - csdfg_init_argv: Arguments used for initialization; used only the first time and
+    #       never updated.
+    csdfg_argv: MutableSequence[Any] | None
+    csdfg_init_argv: Sequence[Any] | None
+
     def __init__(
         self,
         program: dace.CompiledSDFG,
@@ -56,12 +65,46 @@ class CompiledDaceProgram(stages.CompiledProgram):
         # For debug purpose, we set a unique module name on the compiled function.
         self.update_sdfg_ctype_arglist.__module__ = os.path.basename(program.sdfg.build_folder)
 
-    def __call__(self, **kwargs: Any) -> None:
-        result = self.sdfg_program(**kwargs)
-        assert result is None
+        # Since the SDFG hasn't been called yet.
+        self.csdfg_argv = None
+        self.csdfg_init_argv = None
+
+    def construct_arguments(self, **kwargs: Any) -> None:
+        """
+        This function will process the arguments and store the processed argument
+        vectors in `self.csdfg_args`, to call them use `self.fast_call()`.
+        """
+        with dace.config.set_temporary("compiler", "allow_view_arguments", value=True):
+            csdfg_argv, csdfg_init_argv = self.sdfg_program.construct_arguments(**kwargs)
+        # Note we only care about `csdfg_argv` (normal call), since we have to update it,
+        #  we ensure that it is a `list`.
+        self.csdfg_argv = [*csdfg_argv]
+        self.csdfg_init_argv = csdfg_init_argv
 
     def fast_call(self) -> None:
-        result = self.sdfg_program.fast_call(*self.sdfg_program._lastargs)
+        """
+        Perform a call to the compiled SDFG using the previously generated argument
+        vectors, see `self.construct_arguments()`.
+        """
+        assert self.csdfg_argv is not None and self.csdfg_init_argv is not None, (
+            "Argument vector was not set properly."
+        )
+        self.sdfg_program.fast_call(
+            self.csdfg_argv, self.csdfg_init_argv, do_gpu_check=config.DEBUG
+        )
+
+    def __call__(self, **kwargs: Any) -> None:
+        """Call the compiled SDFG with the given arguments.
+
+        Note that this function will not update the argument vectors stored inside
+        `self`. Furthermore, it is not recommended to use this function as it is
+        very slow.
+        """
+        warnings.warn(
+            "Called an SDFG through the standard DaCe interface is not recommended, use `fast_call()` instead.",
+            stacklevel=1,
+        )
+        result = self.sdfg_program(**kwargs)
         assert result is None
 
 
