@@ -11,7 +11,6 @@ from __future__ import annotations
 import functools
 from typing import Any, Sequence
 
-import dace
 import numpy as np
 
 from gt4py._core import definitions as core_defs
@@ -45,15 +44,17 @@ def convert_args(
             args = (*args, out)
 
         try:
-            # Initialization of `_lastargs` was done by the `CompiledSDFG` object,
-            #  so we just update it with the current call arguments.
-            update_sdfg_call_args(args, fun.sdfg_program._lastargs[0])
-            fun.fast_call()
-        except IndexError:
-            # First call, the SDFG is not initialized, so forward the call to `CompiledSDFG`
-            # to properly initialize it. Later calls to this SDFG will be handled through
-            # the `fast_call()` API.
-            assert len(fun.sdfg_program._lastargs) == 0  # `fun.sdfg_program._lastargs` is empty
+            # Not the first call.
+            #  We will only update the argument vector  for the normal call.
+            # NOTE: If this is the first time then we will generate an exception because
+            #   `fun.csdfg_args` is `None`
+            # TODO(phimuell, edopao): Think about refactor the code such that the update
+            #   of the argument vector is a Method of the `CompiledDaceProgram`.
+            update_sdfg_call_args(args, fun.csdfg_argv)  # type: ignore[arg-type]  # Will error out in first call.
+
+        except TypeError:
+            # First call. Construct the initial argument vector of the `CompiledDaceProgram`.
+            assert fun.csdfg_argv is None and fun.csdfg_init_argv is None
             flat_args: Sequence[Any] = gtx_utils.flatten_nested_tuple(args)
             this_call_args = sdfg_callable.get_sdfg_args(
                 fun.sdfg_program.sdfg,
@@ -65,8 +66,10 @@ def convert_args(
                 gtx_wfdcommon.SDFG_ARG_METRIC_LEVEL: config.COLLECT_METRICS_LEVEL,
                 gtx_wfdcommon.SDFG_ARG_METRIC_COMPUTE_TIME: collect_time_arg,
             }
-            with dace.config.set_temporary("compiler", "allow_view_arguments", value=True):
-                fun(**this_call_args)
+            fun.construct_arguments(**this_call_args)
+
+        # Perform the call to the SDFG.
+        fun.fast_call()
 
         if collect_time:
             metric_source = metrics.get_current_source()
